@@ -444,39 +444,35 @@ format_time (struct tm   *now,
 
 	if (local_now.tm_wday != now->tm_wday) {
 		if (clock_format == CLOCK_FORMAT_12) {
-/* translators: This is a strftime format string that is used for formatting 
- * the time that is displayed for locations in the popup. This string is used
- * for 12-hour format (the %p expands to am/pm), when the local weekday differs
- * from the weekday at the location (the %A expands to the weekday).
- * There should be little need to translate this string.
- */
+			/* Translators: This is a strftime format string.
+			 * It is used to display the time in 12-hours format
+			 * (eg, like in the US: 8:10 am), when the local
+			 * weekday differs from the weekday at the location
+			 * (the %A expands to the weekday). The %p expands to
+			 * am/pm. */
 			format = _("%l:%M <small>%p (%A)</small>");
 		}
 		else {
-/* translators: This is a strftime format string that is used for formatting 
- * the time that is displayed for locations in the popup. This string is used
- * for 24-hour format (the %p expands to am/pm), when the local weekday differs
- * from the weekday at the location (the %A expands to the weekday).
- * There should be little need to translate this string.
- */
+			/* Translators: This is a strftime format string.
+			 * It is used to display the time in 24-hours format
+			 * (eg, like in France: 20:10), when the local
+			 * weekday differs from the weekday at the location
+			 * (the %A expands to the weekday). */
 			format = _("%H:%M <small>(%A)</small>");
 		}
 	}
 	else {
 		if (clock_format == CLOCK_FORMAT_12) {
-/* translators: This is a strftime format string that is used for formatting 
- * the time that is displayed for locations in the popup. This string is used
- * for 12-hour format (the %p expands to am/pm). 
- * There should be little need to translate this string.
- */
+			/* Translators: This is a strftime format string.
+			 * It is used to display the time in 12-hours format
+			 * (eg, like in the US: 8:10 am). The %p expands to
+			 * am/pm. */
 			format = _("%l:%M <small>%p</small>");
 		}
 		else {
-/* translators: This is a strftime format string that is used for formatting 
- * the time that is displayed for locations in the popup. This string is used
- * for 24-hour format.
- * There should be little need to translate this string.
- */
+			/* Translators: This is a strftime format string.
+			 * It is used to display the time in 24-hours format
+			 * (eg, like in France: 20:10). */
 			format = _("%H:%M");
 		}
 	}
@@ -505,6 +501,34 @@ format_time (struct tm   *now,
 	return utf8;
 }
 
+static char *
+convert_time_to_str (time_t now, ClockFormat clock_format)
+{
+	const gchar *format;
+	struct tm *tm;
+	gchar buf[128];
+
+	if (clock_format == CLOCK_FORMAT_12) {
+                /* Translators: This is a strftime format string.
+                 * It is used to display the time in 12-hours format (eg, like
+                 * in the US: 8:10 am). The %p expands to am/pm.
+                 */
+		format = _("%l:%M %p");
+	}
+	else {
+                /* Translators: This is a strftime format string.
+                 * It is used to display the time in 24-hours format (eg, like
+                 * in France: 20:10).
+                 */
+		format = _("%H:%M");
+	}
+
+	tm = localtime (&now);
+	strftime (buf, sizeof (buf) - 1, format, tm);
+
+	return g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+}
+
 void
 clock_location_tile_refresh (ClockLocationTile *this, gboolean force_refresh)
 {
@@ -522,7 +546,7 @@ clock_location_tile_refresh (ClockLocationTile *this, gboolean force_refresh)
 		gtk_widget_show (priv->current_marker);
 	}
 	else {
-		if (GTK_WIDGET_VISIBLE (priv->current_marker)) {
+		if (gtk_widget_get_visible (priv->current_marker)) {
 			gtk_widget_hide (priv->current_marker);
 			gtk_widget_hide (priv->current_button);
 			gtk_widget_show (priv->current_spacer);
@@ -544,7 +568,7 @@ clock_location_tile_refresh (ClockLocationTile *this, gboolean force_refresh)
 	priv->last_offset = clock_location_get_offset (priv->location);
 
         tmp = g_strdup_printf ("<big><b>%s</b></big>",
-                               clock_location_get_name (priv->location));
+                               clock_location_get_display_name (priv->location));
         gtk_label_set_markup (GTK_LABEL (priv->city_label), tmp);
         g_free (tmp);
 
@@ -560,7 +584,8 @@ clock_location_tile_refresh (ClockLocationTile *this, gboolean force_refresh)
 }
 
 void
-weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkTooltip *tooltip)
+weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkTooltip *tooltip,
+			    ClockFormat clock_format)
 {
         GdkPixbuf *pixbuf = NULL;
         GtkIconTheme *theme = NULL;
@@ -569,10 +594,13 @@ weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkToolt
 	gchar *line1, *line2, *line3, *line4, *tip;
 	const gchar *icon_name;
 	const gchar *sys_timezone;
+	time_t sunrise_time, sunset_time;
+	gchar *sunrise_str, *sunset_str;
 
        	icon_name = weather_info_get_icon_name (info);
         theme = gtk_icon_theme_get_default ();
-        pixbuf = gtk_icon_theme_load_icon (theme, icon_name, 48, 0, NULL);
+        pixbuf = gtk_icon_theme_load_icon (theme, icon_name, 48,
+                                           GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
         if (pixbuf)
                 gtk_tooltip_set_icon (tooltip, pixbuf);
 
@@ -591,6 +619,7 @@ weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkToolt
 	if (strcmp (apparent, temp) != 0 &&
 	    /* FMQ: it's broken to read from another module's translations; add some API to libgweather. */
             strcmp (apparent, dgettext ("gnome-applets-2.0", "Unknown")) != 0)
+		/* Translators: The two strings are temperatures. */
 		line2 = g_strdup_printf (_("%s, feels like %s"), temp, apparent);
 	else
 		line2 = g_strdup (temp);
@@ -603,15 +632,22 @@ weather_info_setup_tooltip (WeatherInfo *info, ClockLocation *location, GtkToolt
 	else
 		line3 = g_strdup ("");
 
-	/* FIXME: we need libgweather to give us the time,
-	 * not just a formatted string
-	 */
 	sys_timezone = getenv ("TZ");
 	setenv ("TZ", clock_location_get_timezone (location), 1);
 	tzset ();
+	if (weather_info_get_value_sunrise (info, &sunrise_time))
+		sunrise_str = convert_time_to_str (sunrise_time, clock_format);
+	else
+		sunrise_str = g_strdup ("???");
+	if (weather_info_get_value_sunset (info, &sunset_time))
+		sunset_str = convert_time_to_str (sunset_time, clock_format);
+	else
+		sunset_str = g_strdup ("???");
 	line4 = g_strdup_printf (_("Sunrise: %s / Sunset: %s"),
-				 weather_info_get_sunrise (info),
-				 weather_info_get_sunset (info));
+				 sunrise_str, sunset_str);
+	g_free (sunrise_str);
+	g_free (sunset_str);
+
 	if (sys_timezone)
 		setenv ("TZ", sys_timezone, 1);
 	else
@@ -638,13 +674,16 @@ weather_tooltip (GtkWidget  *widget,
         ClockLocationTile *tile = data;
         ClockLocationTilePrivate *priv = PRIVATE (tile);
 	WeatherInfo *info;
+	int clock_format;
 
 	info = clock_location_get_weather_info (priv->location);
 
 	if (!info || !weather_info_is_valid (info))
 		return FALSE;
 
-	weather_info_setup_tooltip (info, priv->location, tooltip);
+	g_signal_emit (tile, signals[NEED_CLOCK_FORMAT], 0, &clock_format);
+
+	weather_info_setup_tooltip (info, priv->location, tooltip, clock_format);
 
 	return TRUE;
 }
@@ -663,7 +702,8 @@ update_weather_icon (ClockLocation *loc, WeatherInfo *info, gpointer data)
 
         icon_name = weather_info_get_icon_name (info);
         theme = gtk_icon_theme_get_default ();
-        pixbuf = gtk_icon_theme_load_icon (theme, icon_name, 16, 0, NULL);
+        pixbuf = gtk_icon_theme_load_icon (theme, icon_name, 16,
+                                           GTK_ICON_LOOKUP_GENERIC_FALLBACK, NULL);
 
         if (pixbuf) {
                 gtk_image_set_from_pixbuf (GTK_IMAGE (priv->weather_icon), pixbuf);
