@@ -25,20 +25,21 @@
  */
 
 #include <config.h>
+
+#include <math.h>
 #include <string.h>
 #include <time.h>
 
-#include <math.h>
 #include <cairo.h>
+
+#include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-help.h>
-#include <libgnomeui/gnome-uidefs.h>
-#include <glade/glade-xml.h>
 
 #define FISH_APPLET(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), \
 			fish_applet_get_type(),          \
@@ -143,33 +144,56 @@ static void
 show_help (FishApplet *fish, const char *link_id)
 {
 	GError *error = NULL;
+	char   *uri;
+#define FISH_HELP_DOC "fish"
 
-	gnome_help_display_desktop_on_screen (
-		NULL, "fish", "fish", link_id,
-		gtk_widget_get_screen (GTK_WIDGET (fish)),
-		&error);
+	if (link_id)
+		uri = g_strdup_printf ("ghelp:%s?%s", FISH_HELP_DOC, link_id);
+	else
+		uri = g_strdup_printf ("ghelp:%s", FISH_HELP_DOC);
 
-	if (error) {
+	gtk_show_uri (gtk_widget_get_screen (GTK_WIDGET (fish)), uri,
+		      gtk_get_current_event_time (), &error);
+
+	g_free (uri);
+
+	if (error &&
+	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+		g_error_free (error);
+	else if (error) {
 		GtkWidget *dialog;
+		char      *primary;
 
+		primary = g_markup_printf_escaped (
+				_("Could not display help document '%s'"),
+				FISH_HELP_DOC);
 		dialog = gtk_message_dialog_new (
 				NULL,
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_ERROR,
-				GTK_BUTTONS_OK,
-				_("There was an error displaying help: %s"),
-				error->message);
+				GTK_BUTTONS_CLOSE,
+				"%s", primary);
+
+		gtk_message_dialog_format_secondary_text (
+					GTK_MESSAGE_DIALOG (dialog),
+					"%s", error->message);
+
+		g_error_free (error);
+		g_free (primary);
 
 		g_signal_connect (dialog, "response",
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
 
 		gtk_window_set_icon_name (GTK_WINDOW (dialog), FISH_ICON);
-		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		gtk_window_set_screen (GTK_WINDOW (dialog),
 				       gtk_widget_get_screen (GTK_WIDGET (fish)));
+		/* we have no parent window */
+		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+		gtk_window_set_title (GTK_WINDOW (dialog),
+				      _("Error displaying help document"));
+
 		gtk_widget_show (dialog);
-		g_error_free (error);
 	}
 }
 
@@ -321,7 +345,7 @@ handle_response (GtkWidget  *widget,
 
 static void
 setup_sensitivity (FishApplet *fish,
-		   GladeXML *xml,
+		   GtkBuilder *builder,
 		   const char *wid,
 		   const char *label,
 		   const char *label_post,
@@ -339,17 +363,17 @@ setup_sensitivity (FishApplet *fish,
 	}
 	g_free (fullkey);
 
-	w = glade_xml_get_widget (xml, wid);
+	w = GTK_WIDGET (gtk_builder_get_object (builder, wid));
 	g_assert (w != NULL);
 	gtk_widget_set_sensitive (w, FALSE);
 
 	if (label != NULL) {
-		w = glade_xml_get_widget (xml, label);
+		w = GTK_WIDGET (gtk_builder_get_object (builder, label));
 		g_assert (w != NULL);
 		gtk_widget_set_sensitive (w, FALSE);
 	}
 	if (label_post != NULL) {
-		w = glade_xml_get_widget (xml, label_post);
+		w = GTK_WIDGET (gtk_builder_get_object (builder, label_post));
 		g_assert (w != NULL);
 		gtk_widget_set_sensitive (w, FALSE);
 	}
@@ -388,7 +412,8 @@ display_preferences_dialog (BonoboUIComponent *uic,
 			    FishApplet        *fish,
 			    const char        *verbname)
 {
-	GladeXML      *xml;
+	GtkBuilder    *builder;
+	GError        *error;
 	GtkWidget     *button;
 	GtkFileFilter *filter;
 	GtkWidget     *chooser_preview;
@@ -401,8 +426,18 @@ display_preferences_dialog (BonoboUIComponent *uic,
 		return;
 	}
 
-	xml = glade_xml_new (FISH_GLADEDIR "/fish.glade", NULL, NULL);
-	fish->preferences_dialog = glade_xml_get_widget (xml, "fish_preferences_dialog");
+	builder = gtk_builder_new ();
+	gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
+
+	error = NULL;
+	gtk_builder_add_from_file (builder, FISH_BUILDERDIR "/fish.ui", &error);
+	if (error) {
+		g_warning ("Error loading preferences: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	fish->preferences_dialog = GTK_WIDGET (gtk_builder_get_object (builder, "fish_preferences_dialog"));
 
 	g_object_add_weak_pointer (G_OBJECT (fish->preferences_dialog),
 				   (void**) &fish->preferences_dialog);
@@ -414,24 +449,24 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	gtk_dialog_set_default_response (
 		GTK_DIALOG (fish->preferences_dialog), GTK_RESPONSE_OK);
 
-	fish->name_entry = glade_xml_get_widget (xml, "name_entry");
+	fish->name_entry = GTK_WIDGET (gtk_builder_get_object (builder, "name_entry"));
 	gtk_entry_set_text (GTK_ENTRY (fish->name_entry), fish->name);
 
 	g_signal_connect (fish->name_entry, "changed",
 			  G_CALLBACK (name_value_changed), fish);
 
-	setup_sensitivity (fish, xml,
+	setup_sensitivity (fish, builder,
 			   "name_entry" /* wid */,
 			   "name_label" /* label */,
 			   NULL /* label_post */,
 			   "name" /* key */);
 
-	fish->preview_image = glade_xml_get_widget (xml, "preview_image");
+	fish->preview_image = GTK_WIDGET (gtk_builder_get_object (builder, "preview_image"));
 	if (fish->pixbuf)
 		gtk_image_set_from_pixbuf (GTK_IMAGE (fish->preview_image),
 					   fish->pixbuf);
 
-	fish->image_chooser =  glade_xml_get_widget (xml, "image_chooser");
+	fish->image_chooser =  GTK_WIDGET (gtk_builder_get_object (builder, "image_chooser"));
 	filter = gtk_file_filter_new ();
 	gtk_file_filter_set_name (filter, _("Images"));
 	gtk_file_filter_add_pixbuf_formats (filter);
@@ -452,20 +487,20 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	g_signal_connect (fish->image_chooser, "selection-changed",
 			  G_CALLBACK (image_value_changed), fish);
 
-	setup_sensitivity (fish, xml,
+	setup_sensitivity (fish, builder,
 			   "image_chooser" /* wid */,
 			   "image_label" /* label */,
 			   NULL /* label_post */,
 			   "image" /* key */);
 
-	fish->command_label = glade_xml_get_widget (xml, "command_label");
-	fish->command_entry = glade_xml_get_widget (xml, "command_entry");
+	fish->command_label = GTK_WIDGET (gtk_builder_get_object (builder, "command_label"));
+	fish->command_entry = GTK_WIDGET (gtk_builder_get_object (builder, "command_entry"));
 	gtk_entry_set_text (GTK_ENTRY (fish->command_entry), fish->command);
 
 	g_signal_connect (fish->command_entry, "changed",
 			  G_CALLBACK (command_value_changed), fish);
 
-	setup_sensitivity (fish, xml,
+	setup_sensitivity (fish, builder,
 			   "command_entry" /* wid */,
 			   "command_label" /* label */,
 			   NULL /* label_post */,
@@ -478,39 +513,39 @@ display_preferences_dialog (BonoboUIComponent *uic,
 		gtk_widget_set_sensitive (fish->command_entry, FALSE);
 	}
 
-	fish->frames_spin = glade_xml_get_widget (xml, "frames_spin");
+	fish->frames_spin = GTK_WIDGET (gtk_builder_get_object (builder, "frames_spin"));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (fish->frames_spin),
 				   fish->n_frames);
 
 	g_signal_connect (fish->frames_spin, "value_changed",
 			  G_CALLBACK (n_frames_value_changed), fish);
 
-	setup_sensitivity (fish, xml,
+	setup_sensitivity (fish, builder,
 			   "frames_spin" /* wid */,
 			   "frames_label" /* label */,
 			   "frames_post_label" /* label_post */,
 			   "frames" /* key */);
 
-	fish->speed_spin = glade_xml_get_widget (xml, "speed_spin");
+	fish->speed_spin = GTK_WIDGET (gtk_builder_get_object (builder, "speed_spin"));
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (fish->speed_spin), fish->speed);
 
 	g_signal_connect (fish->speed_spin, "value_changed",
 			  G_CALLBACK (speed_value_changed), fish);
 
-	setup_sensitivity (fish, xml,
+	setup_sensitivity (fish, builder,
 			   "speed_spin" /* wid */,
 			   "speed_label" /* label */,
 			   "speed_post_label" /* label_post */,
 			   "speed" /* key */);
 
-	fish->rotate_toggle = glade_xml_get_widget (xml, "rotate_toggle");
+	fish->rotate_toggle = GTK_WIDGET (gtk_builder_get_object (builder, "rotate_toggle"));
 	gtk_toggle_button_set_active (
 		GTK_TOGGLE_BUTTON (fish->rotate_toggle), fish->rotate);
 
 	g_signal_connect (fish->rotate_toggle, "toggled",
 			  G_CALLBACK (rotate_value_changed), fish);
 
-	setup_sensitivity (fish, xml,
+	setup_sensitivity (fish, builder,
 			   "rotate_toggle" /* wid */,
 			   NULL /* label */,
 			   NULL /* label_post */,
@@ -521,7 +556,7 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	g_signal_connect (fish->preferences_dialog, "response",
 			  G_CALLBACK (handle_response), fish);
 
-	button = glade_xml_get_widget (xml, "done_button");
+	button = GTK_WIDGET (gtk_builder_get_object (builder, "done_button"));
         g_signal_connect_swapped (button, "clicked",
 				  (GCallback) gtk_widget_hide, 
 				  fish->preferences_dialog);
@@ -531,7 +566,7 @@ display_preferences_dialog (BonoboUIComponent *uic,
 	gtk_window_set_resizable (GTK_WINDOW (fish->preferences_dialog), FALSE);
 	gtk_window_present (GTK_WINDOW (fish->preferences_dialog));
 
-	g_object_unref (xml);
+	g_object_unref (builder);
 }
 
 static void
@@ -579,7 +614,7 @@ display_about_dialog (BonoboUIComponent *uic,
 		
 	fish->about_dialog = gtk_about_dialog_new ();
 	g_object_set (fish->about_dialog,
-		      "name", _("Fish"),
+		      "program-name", _("Fish"),
 		      "version", "3.4.7.4ac19",
 		      "copyright", "Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
 		      "comments", descr,
@@ -641,8 +676,8 @@ something_fishy_going_on (FishApplet *fish,
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK,
-					 message);
+					 GTK_BUTTONS_CLOSE,
+					 "%s", message);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (gtk_widget_destroy),
@@ -926,15 +961,15 @@ display_fortune_dialog (FishApplet *fish)
 
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
 				    fish->fortune_label,
-				    FALSE, FALSE, GNOME_PAD);
+				    FALSE, FALSE, 6);
 
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
 				    scrolled,
-				    TRUE, TRUE, GNOME_PAD);
+				    TRUE, TRUE, 6);
 
 		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
 				    fish->fortune_cmd_label,
-				    FALSE, FALSE, GNOME_PAD);
+				    FALSE, FALSE, 6);
 
 		update_fortune_dialog (fish);
 
@@ -950,7 +985,7 @@ display_fortune_dialog (FishApplet *fish)
 		char *command;
 		char * text;
 
-		command = g_strdup_printf ("<tt>%s</tt>", argv[0]);
+		command = g_markup_printf_escaped ("<tt>%s</tt>", argv[0]);
 		text = g_strdup_printf (_("The configured command is not "
 					  "working and has been replaced by: "
 					  "%s"), command);
@@ -1134,31 +1169,38 @@ static char *
 get_location (void)
 {
 	static char  location [256];
-	char         buffer [256];
+	char        *buffer;
 	FILE        *zone;
 	int          i, len, count;
 	
 	/* Old method : works for glibc < 2.2 */
 	zone = fopen("/etc/timezone", "r");
 	if (zone) {
-		fscanf (zone, "%255s", location);
+		count = fscanf (zone, "%255s", location);
 		fclose (zone);
-		return location;
+		/* if we could read it, we return what we got */
+		if (count == 1)
+			return location;
 	} 
 
 	/* New method : works for glibc 2.2 */
-	len = readlink ("/etc/localtime", buffer, sizeof (buffer));
-	if (len <= 0)
+	/* FIXME: this is broken for many distros, see the clock code */
+	buffer = g_file_read_link ("/etc/localtime", NULL);
+	if (!buffer)
 		return NULL;
 
+	len = strlen (buffer);
 	for (i = len, count = 0; (i > 0) && (count != 2); i--)
 		if (buffer [i] == '/')
 			count++;
 
-	if (count != 2)
+	if (count != 2) {
 		return NULL;
+		g_free (buffer);
+	}
 
 	memcpy (location, &buffer [i + 2], len - i - 2);
+	g_free (buffer);
 
 	return location;
 }
@@ -1426,7 +1468,7 @@ update_pixmap (FishApplet *fish)
 	cairo_matrix_t matrix;
 	cairo_pattern_t *pattern;
 
-	if (!GTK_WIDGET_REALIZED (widget) ||
+	if (!gtk_widget_get_realized (widget) ||
 	    widget->allocation.width <= 0 ||
 	    widget->allocation.height <= 0)
 		return;
@@ -1703,36 +1745,14 @@ handle_button_release (FishApplet     *fish,
 static void
 set_tooltip (FishApplet *fish)
 {
-	GtkTooltips *tooltips;
-	const char  *desc_format = _("%s the Fish, the fortune teller");
-	char        *desc;
+	const char *desc_format = _("%s the Fish, the fortune teller");
+	char       *desc;
 
-	tooltips = g_object_get_data (G_OBJECT (fish), "tooltips");
-	if (!tooltips) {
-		tooltips = gtk_tooltips_new ();
-		g_object_ref (tooltips);
-		gtk_object_sink (GTK_OBJECT (tooltips));
-
-		g_object_set_data (G_OBJECT (fish), "tooltips", tooltips);
-	}
-
-	desc = g_strdup_printf (desc_format, fish->name);
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (fish), desc, NULL);
+	desc = g_markup_printf_escaped (desc_format, fish->name);
+	gtk_widget_set_tooltip_markup (GTK_WIDGET (fish), desc);
 	g_free (desc);
 }
 
-static void
-destroy_tooltip (FishApplet *fish)
-{
-	GtkTooltips *tooltips;
-
-	tooltips = g_object_get_data (G_OBJECT (fish), "tooltips");
-	if (tooltips) {
-		g_object_unref (tooltips);
-		g_object_set_data (G_OBJECT (fish), "tooltips", NULL);
-	}
-}
-
 static void
 setup_fish_widget (FishApplet *fish)
 {
@@ -1971,8 +1991,6 @@ fish_applet_destroy (GtkObject *object)
 
 	fish_close_channel (fish);
 
-	destroy_tooltip (fish);
-
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
