@@ -254,7 +254,7 @@ queue_drawer_close_for_drag (Drawer *drawer)
 {
 	if (!drawer->close_timeout_id)
 		drawer->close_timeout_id =
-			g_timeout_add (1 * 1000, close_drawer_in_idle, drawer);
+			g_timeout_add_seconds (1, close_drawer_in_idle, drawer);
 }
 
 static void
@@ -316,7 +316,7 @@ set_tooltip_and_name (Drawer     *drawer,
 		tooltip = NULL;
 
 	panel_toplevel_set_name (drawer->toplevel, tooltip);
-	gtk_tooltips_set_tip (panel_tooltips, drawer->button, tooltip, NULL);
+	panel_util_set_tooltip_text (drawer->button, tooltip);
 }
 
 static Drawer *
@@ -419,7 +419,7 @@ drawer_button_size_allocated (GtkWidget     *widget,
 			      GtkAllocation *alloc,
 			      Drawer        *drawer)
 {
-	if (!GTK_WIDGET_REALIZED (widget))
+	if (!gtk_widget_get_realized (widget))
 		return;
 
 	gtk_widget_queue_resize (GTK_WIDGET (drawer->toplevel));
