@@ -140,7 +140,7 @@ panel_menu_bar_init (PanelMenuBar *menubar)
 
 	menubar->priv->info = NULL;
 
-	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL);
+	menubar->priv->applications_menu = create_applications_menu ("applications.menu", NULL, TRUE);
 
 	menubar->priv->applications_item = panel_image_menu_item_new ();
 	gtk_menu_item_set_label (GTK_MENU_ITEM (menubar->priv->applications_item),
@@ -306,7 +306,7 @@ panel_menu_bar_on_expose (GtkWidget      *widget,
 {
 	PanelMenuBar *menubar = data;
 
-	if (GTK_WIDGET_HAS_FOCUS (menubar))
+	if (gtk_widget_has_focus (GTK_WIDGET (menubar)))
 		gtk_paint_focus (widget->style,
 				 widget->window, 
 				 GTK_WIDGET_STATE (menubar),
